from __future__ import absolute_import
from __future__ import division
from __future__ import print_function
from __future__ import unicode_literals

import logging
import os
import uuid

import amulet
import charmhelpers.contrib.amulet.deployment as ccad
import charmhelpers.contrib.amulet.utils


class MidonetGatewayBaseDeployment(ccad.AmuletDeployment):  # noqa pylint: disable=too-many-instance-attributes
    def __init__(self,
                 series,
                 openstack_release,
                 midonet_flavor,
                 midonet_release):
        super(MidonetGatewayBaseDeployment, self).__init__(series=series)

        self.openstack_release = openstack_release
        self.midonet_flavor = midonet_flavor
        self.midonet_release = midonet_release

        self.check_enterprise()

        self.amulet_utils = charmhelpers.contrib.amulet.utils.AmuletUtils(
            log_level=logging.DEBUG,
        )

        self.add_services()
        self.add_relations()
        self.configure_services()
        self.deploy()

    def check_enterprise(self):
        if self.midonet_flavor == 'enterprise':
            try:
                self.midonet_enterprise_username = os.environ[
                    'MIDONET_ENTERPRISE_USERNAME'
                ]
                self.midonet_enterprise_password = os.environ[
                    'MIDONET_ENTERPRISE_PASSWORD'
                ]
            except KeyError:
                amulet.raise_status(
                    amulet.SKIP,
                    msg=(
                        'Skipping this permutation since username and/or '
                        'passwod for Midokura Enterprise Midonet was not '
                        'supplied.'
                    ),
                )

    def add_services(self):
        self.amulet_utils.log.info('Adding services...')

        this_service = {'name': 'midonet-gateway'}
        this_service.update(getattr(self, 'service_options', {}))

        other_services = [
            {'name': 'mysql'},
            {'name': 'rabbitmq-server'},
            {'name': 'keystone'},
            {'name': 'glance'},
            {'name': 'nova-cloud-controller'},
            {'name': 'nova-compute'},
            {'name': 'neutron-api',
             'location': 'cs:~openstack-charmers-next/trusty/neutron-api'},
            {'name': 'zookeeper'},
            {'name': 'cassandra'},
            {'name': 'midonet-api',
             'location': 'lp:~ryo-tagami/charms/trusty/midonet-api/trunk'},
            {'name': 'neutron-agents-midonet'},
            {'name': 'midonet-agent'},
        ]

        for service in other_services:
            if 'location' not in service:
                service['location'] = 'cs:{}/{}'.format(
                    self.series,
                    service['name'],
                )

        super(MidonetGatewayBaseDeployment, self)._add_services(
            this_service,
            other_services,
        )

    def add_relations(self):
        self.amulet_utils.log.info('Adding relations...')

        relations = {
            'keystone:shared-db': 'mysql:shared-db',
            'glance:shared-db': 'mysql:shared-db',
            'glance:amqp': 'rabbitmq-server:amqp',
            'glance:identity-service': 'keystone:identity-service',
            'nova-cloud-controller:shared-db': 'mysql:shared-db',
            'nova-cloud-controller:amqp': 'rabbitmq-server:amqp',
            'nova-cloud-controller:identity-service': 'keystone:identity-service',  # noqa pylint: disable=line-too-long
            'nova-cloud-controller:image-service': 'glance:image-service',
            'nova-compute:amqp': 'rabbitmq-server:amqp',
            'nova-compute:image-service': 'glance:image-service',
            'nova-compute:cloud-compute': 'nova-cloud-controller:cloud-compute',  # noqa pylint: disable=line-too-long
            'neutron-api:shared-db': 'mysql:shared-db',
            'neutron-api:amqp': 'rabbitmq-server:amqp',
            'neutron-api:identity-service': 'keystone:identity-service',
            'neutron-api:neutron-api': 'nova-cloud-controller:neutron-api',
            'midonet-api:keystone': 'keystone:identity-service',
            'midonet-api:midonet-api': 'neutron-api:midonet',
            'midonet-api:zookeeper': 'zookeeper:zookeeper',
            'midonet-api:host': 'midonet-agent:host',
            'neutron-agents-midonet:amqp': 'rabbitmq-server:amqp',
            'neutron-agents-midonet:neutron_agents': 'nova-cloud-controller:quantum-network-service',  # noqa pylint: disable=line-too-long
            'neutron-agents-midonet:neutron-plugin-api': 'neutron-api:neutron-plugin-api',  # noqa pylint: disable=line-too-long
            'neutron-agents-midonet:midonet': 'midonet-api:midonet-api',
            'neutron-agents-midonet:agent': 'midonet-agent:host',
            'midonet-agent:neutron-plugin': 'nova-compute:neutron-plugin',
            'midonet-agent:zookeeper': 'zookeeper:zookeeper',
            'midonet-agent:cassandra': 'cassandra:database',
            'midonet-gateway:identity-admin': 'keystone:identity-admin',
            'midonet-gateway:neutron-plugin-api': 'neutron-api:neutron-plugin-api',  # noqa pylint: disable=line-too-long
            'midonet-gateway:midonet-api': 'midonet-api:midonet-api',
            'midonet-gateway:midonet-host': 'midonet-agent:host',
        }

        super(MidonetGatewayBaseDeployment, self)._add_relations(relations)

    def configure_services(self):
        self.amulet_utils.log.info('Configuring services...')

        mysql = {
            'max-connections': 2048,
        }
        keystone = {
            'admin-password': 'password',
            'admin-token': 'token',
        }
        nova_cloud_controller = {
            'network-manager': 'Neutron',
        }
        nova_compute = {
            'manage-neutron-plugin-legacy-mode': False,
            'virt-type': 'qemu',
        }
        neutron_api = {
            'l2-population': False,
            'neutron-plugin': 'midonet',
            'neutron-security-groups': True,
        }
        cassandra = {
            'authenticator': 'AllowAllAuthenticator',
            'cluster_name': 'midonet',
            'extra_packages': 'dsc20',
            'install_sources': '\n'.join([
                '- deb http://debian.datastax.com/community 2.0 main',
                '- ppa:openjdk-r/ppa',
                '- ppa:stub/cassandra',
            ]),
        }

        midonet_gateway = {
            'openstack-release': self.openstack_release,
            'midonet-flavor': self.midonet_flavor,
            'midonet-release': self.midonet_release,
        }
        if midonet_gateway['midonet-flavor'] == 'enterprise':
            midonet_gateway['midonet-enterprise-username'] = (
                self.midonet_enterprise_username
            )
            midonet_gateway['midonet-enterprise-password'] = (
                self.midonet_enterprise_password
            )
        midonet_gateway.update(getattr(self, 'service_configurations', {}))

        configs = {
            'mysql': mysql,
            'keystone': keystone,
            'nova-cloud-controller': nova_cloud_controller,
            'nova-compute': nova_compute,
            'neutron-api': neutron_api,
            'cassandra': cassandra,
            'midonet-gateway': midonet_gateway,
        }

        charms_with_openstack_origin = [
            'keystone',
            'glance',
            'nova-cloud-controller',
            'nova-compute',
            'neutron-api',
            'midonet-api',
            'neutron-agents-midonet',
            'midonet-agent',
        ]
        for charm in charms_with_openstack_origin:
            if charm not in configs:
                configs[charm] = {}

            configs[charm]['openstack-origin'] = 'cloud:{}-{}'.format(
                self.series,
                self.openstack_release,
            )

        charms_with_midonet_origin = [
            'neutron-api',
            'midonet-api',
            'neutron-agents-midonet',
            'midonet-agent',
        ]
        short_midonet_flavor_name_dict = {
            'enterprise': 'mem',
            'opensource': 'midonet',
        }
        for charm in charms_with_midonet_origin:
            if charm not in configs:
                configs[charm] = {}

            configs[charm]['midonet-origin'] = '{}-{}'.format(
                short_midonet_flavor_name_dict[self.midonet_flavor],
                self.midonet_release,
            )

            if self.midonet_flavor == 'enterprise':
                configs[charm]['mem-username'] = (
                    self.midonet_enterprise_username
                )
                configs[charm]['mem-password'] = (
                    self.midonet_enterprise_password
                )

        super(MidonetGatewayBaseDeployment, self)._configure_services(configs)

    def deploy(self):
        try:
            self.amulet_utils.log.info('Deploying ...')
            self.d.setup(timeout=7200)
            self.amulet_utils.log.info('Executing hooks ...')
            self.d.sentry.wait(timeout=7200)
            self.amulet_utils.log.info('Deployment ready.')
        except amulet.helpers.TimeoutError:
            amulet.raise_status(amulet.FAIL, msg="Deployment timed out.")

        self.keystone_sentry = self.d.sentry['keystone'][0]
        self.midonet_api_sentry = self.d.sentry['midonet-api'][0]
        self.midonet_gateway_sentry = self.d.sentry['midonet-gateway'][0]

    def _check_call(self, command, errmsg):
        self.amulet_utils.log.debug('Running command: %s ...', repr(command))

        stdout, exit_status = self.midonet_gateway_sentry.run(command)

        if exit_status != 0:
            amulet.raise_status(amulet.FAIL, msg=errmsg)

        return stdout

    def _check_call_negative(self, command, errmsg):
        self.amulet_utils.log.debug('Running command: %s ...', repr(command))

        stdout, exit_status = self.midonet_gateway_sentry.run(command)

        if exit_status == 0:
            amulet.raise_status(amulet.FAIL, msg=errmsg)

        return stdout

    def _check_output(self, command, output, errmsg):
        self.amulet_utils.log.debug('Running command: %s ...', repr(command))

        stdout, _ = self.midonet_gateway_sentry.run(command)

        if stdout.splitlines()[0].strip() != output:
            amulet.raise_status(amulet.FAIL, msg=errmsg)

    def _check_midonet_cli(self, command, errmsg):
        self.amulet_utils.log.debug('Running command: %s ...', repr(command))

        stdout, _ = self.midonet_gateway_sentry.run(
            "midonet-cli -e '{}'".format(command)
        )

        stdout_lines = stdout.splitlines()

        if len(stdout_lines) != 1:
            amulet.raise_status(amulet.FAIL, msg=errmsg)

        try:
            object_uuid_str = str(uuid.UUID(stdout_lines[0].split()[1]))
        except ValueError:
            amulet.raise_status(amulet.FAIL, msg=errmsg)

        return object_uuid_str
