from __future__ import absolute_import
from __future__ import division
from __future__ import print_function
from __future__ import unicode_literals

import lib.amulet._base_deployment as la_b_d


BGP_ADVERTISED_NETWORKS_YAML = """\
- '192.0.2.128/25'
- '198.51.100.128/25'
- '203.0.113.128/25'
"""
BGP_ADVERTISED_NETWORKS_OBJECT = [
    '192.0.2.128/25',
    '198.51.100.128/25',
    '203.0.113.128/25',
]

BGP_TOPOLOGY_YAML = """\
- local_ip_address: '192.0.2.6/29'
  local_mac_address: '00:00:5e:00:53:01'
  peer_ip_addresses:
    - '192.0.2.1'
    - '192.0.2.2'
    - '192.0.2.3'
- local_ip_address: '198.51.100.6/29'
  local_mac_address: '00:00:5e:00:53:02'
  peer_ip_addresses:
    - '198.51.100.1'
    - '198.51.100.2'
    - '198.51.100.3'
- local_ip_address: '203.0.113.6/29'
  local_mac_address: '00:00:5e:00:53:03'
  peer_ip_addresses:
    - '203.0.113.1'
    - '203.0.113.2'
    - '203.0.113.3'
"""
BGP_TOPOLOGY_OBJECT = [
    {
        'local_ip_address': '192.0.2.6',
        'local_ip_network': '192.0.2.0/29',
        'local_mac_address': '00:00:5e:00:53:01',
        'peer_ip_addresses': [
            '192.0.2.1',
            '192.0.2.2',
            '192.0.2.3',
        ]
    },
    {
        'local_ip_address': '198.51.100.6',
        'local_ip_network': '198.51.100.0/29',
        'local_mac_address': '00:00:5e:00:53:02',
        'peer_ip_addresses': [
            '198.51.100.1',
            '198.51.100.2',
            '198.51.100.3',
        ]
    },
    {
        'local_ip_address': '203.0.113.6',
        'local_ip_network': '203.0.113.0/29',
        'local_mac_address': '00:00:5e:00:53:03',
        'peer_ip_addresses': [
            '203.0.113.1',
            '203.0.113.2',
            '203.0.113.3',
        ]
    },
]


class MidonetGatewayDeployment(la_b_d.MidonetGatewayBaseDeployment):
    def __init__(self,
                 series,
                 openstack_release,
                 midonet_flavor,
                 midonet_release):
        self.service_options = {
            'units': 3,
        }
        self.service_configurations = {
            'uplink-type': 'bgp',
            'bgp-advertised-networks': BGP_ADVERTISED_NETWORKS_YAML,
            'bgp-topology': BGP_TOPOLOGY_YAML,
            'bgp-routing-amulet-test': True,
        }

        super(MidonetGatewayDeployment, self).__init__(
            series=series,
            openstack_release=openstack_release,
            midonet_flavor=midonet_flavor,
            midonet_release=midonet_release,
        )

    def test_host_configuration(self):
        self.amulet_utils.log.info('Verifying host configuration ...')

        for unit_number in [0, 1, 2]:
            interface_name = 'dummy{}'.format(unit_number + 1)

            self._check_call(
                command=(
                    'ip link show {} up'
                    ''.format(interface_name)
                ),
                errmsg=(
                    'Interface {} not found or down.'
                    ''.format(interface_name)
                ),
                # unit_number=unit_number,
            )

            self._check_call(
                command=(
                    'ip link show {} up | grep "master midonet"'
                    ''.format(interface_name)
                ),
                errmsg=(
                    'Interface {} does not belongs to MidoNet.'
                    ''.format(interface_name)
                ),
                # unit_number=unit_number,
            )

        self.amulet_utils.log.info('Host configuration verified.')

    def test_neutron_configuration(self):
        self.amulet_utils.log.info('Verifying Neutron configuration ...')

        keystone_address = self.d.sentry.unit['keystone/0'].relation(
            'identity-admin', 'midonet-gateway:identity-admin',
        )['private-address']

        self._check_call(
            command=(
                'neutron --os-auth-url http://{}:5000/v2.0'
                ' --os-tenant-name Admin --os-username admin'
                ' --os-password password net-show ext-net'.format(
                    keystone_address,
                )
            ),
            errmsg='Neutron network ext-net does not exists.'
        )

        self.amulet_utils.log.info('Neutron configuration verified.')

    def test_midonet_configuration(self):
        self.amulet_utils.log.info('Verifying MidoNet configuration ...')

        midonet_api_address = self.d.sentry.unit['midonet-api/0'].relation(
            'midonet-api', 'midonet-gateway:midonet-api',
        )['private-address']

        for unit_number in [0, 1, 2]:
            unit_topology = BGP_TOPOLOGY_OBJECT[unit_number]

            self._check_call(
                command=(
                    'echo -en "'
                    '[cli]\n'
                    'api_url = http://{}:8080/midonet-api\n'
                    'username = admin\n'
                    'password = password\n'
                    'project_id = Admin\n'
                    '" > ~/.midonetrc'.format(midonet_api_address)
                ),
                errmsg='Could not write .midonetrc .',
                # unit_number=unit_number,
            )

            router = self._check_midonet_cli(
                command='router list name "MidoNet Provider Router"',
                errmsg='Could not find router.',
                # unit_number=unit_number,
            )

            port = self._check_midonet_cli(
                command=(
                    'router {} port list plugged yes address {} net {}'
                    ''.format(
                        router,
                        unit_topology['local_ip_address'],
                        unit_topology['local_ip_network'],
                    )
                ),
                errmsg='Could not find port.',
                # unit_number=unit_number,
            )

            for peer_ip_address in unit_topology['peer_ip_addresses']:
                bgp = self._check_midonet_cli(
                    command=(
                        'router {} port {} list bgp '
                        'local-AS 64512 peer-AS 64513 peer {}'
                        ''.format(router, port, peer_ip_address)
                    ),
                    errmsg='Could not find bgp.',
                    # unit_number=unit_number,
                )

                for advertised_network in BGP_ADVERTISED_NETWORKS_OBJECT:
                    self._check_midonet_cli(
                        command=(
                            'router {} port {} bgp {} list route net {}'
                            ''.format(router, port, bgp, advertised_network)
                        ),
                        errmsg='Could not find ad-route.',
                        # unit_number=unit_number,
                    )

            self._check_midonet_cli(
                command=(
                    'router {} route list '
                    'type normal src 0.0.0.0/0 dst {} port {}'
                    ''.format(router, unit_topology['local_ip_network'], port)
                ),
                errmsg='Could not find route.',
                # unit_number=unit_number,
            )

            host = self._check_call(
                command=(
                    'cat /etc/midonet_host_id.properties'
                    '| grep host_uuid | cut -d "=" -f 2'
                ),
                errmsg=(
                    'Could not read or parse /etc/midonet_host_id.properties .'
                ),
                # unit_number=unit_number,
            )

            self._check_midonet_cli(
                command=(
                    'host {} binding list interface dummy{} port {}'
                    ''.format(host, (unit_number + 1), port)
                ),
                errmsg=(
                    'Could not find binding for port.'
                ),
                # unit_number=unit_number,
            )

            port_group = self._check_midonet_cli(
                command='port-group list name uplink-spg stateful true',
                errmsg='Could not find port-group.',
                # unit_number=unit_number,
            )

            self._check_midonet_cli(
                command=(
                    'port-group {} member list port {}'
                    ''.format(port_group, port)
                ),
                errmsg='Could not find port in port-group.',
                # unit_number=unit_number,
            )

        self.amulet_utils.log.info('MidoNet configuration verified.')
