from __future__ import absolute_import
from __future__ import division
from __future__ import print_function
from __future__ import unicode_literals

import copy

import netaddr

import lib.amulet._base_deployment as la_b_d


class MidonetGatewayDeployment(la_b_d.MidonetGatewayBaseDeployment):
    def __init__(self,
                 series,
                 openstack_release,
                 midonet_flavor,
                 midonet_release):
        super(MidonetGatewayDeployment, self).__init__(
            series=series,
            openstack_release=openstack_release,
            midonet_flavor=midonet_flavor,
            midonet_release=midonet_release,
        )

        self.base_config = {
            'static-linux-bridge-address': '172.19.0.1/30',
            'static-provider-router-address': '172.19.0.2/30',
            'static-forwarding-network': '200.200.200.0/24',
            'static-outgoing-masquerade': True,
            'static-outgoing-interface-name': 'eth0',
        }

    def test_(self):
        self._run_tests(self.base_config)

        self._configure_and_run_tests({
            'static-linux-bridge-address': '172.19.1.1/30',
            'static-provider-router-address': '172.19.1.2/30',
        })

        self._configure_and_run_tests({
            'static-forwarding-network': '200.200.201.0/24',
        })

        self._configure_and_run_tests({
            'static-outgoing-masquerade': False,
        })

        self._configure_and_run_tests({
            'static-outgoing-interface-name': 'eth1',
        })

    def _configure_and_run_tests(self, config_fragment):
        config = copy.deepcopy(self.base_config)
        config.update(config_fragment)
        self.d.configure('midonet-gateway', config)
        self.d.sentry.wait()
        self._run_tests(config)

    def _run_tests(self, config):
        self._test_host_configuration(config)
        self._test_neutron_configuration(config)
        self._test_midonet_configuration(config)

    def _test_host_configuration(self, config):
        self.amulet_utils.log.info('Verifying host configuration ...')

        self._check_call(
            command='ip link show veth0 up',
            errmsg='Interface veth0 not found or down.',
        )

        self._check_call(
            command='ip link show veth1 up',
            errmsg='Interface veth1 not found or down.',
        )

        self._check_call(
            command='ip link show uplinkbridge up',
            errmsg='Bridge uplinkbridge not found or down.',
        )

        self._check_call(
            command='ip link show veth0 up | grep "master uplinkbridge"',
            errmsg='Interface veth0 does not belongs to bridge uplinkbridge.',
        )

        self._check_call(
            command='ip link show veth1 up | grep "master midonet"',
            errmsg='Interface veth1 does not belongs to MidoNet.',
        )

        self._check_call(
            command=(
                'ip -4 -o addr show dev uplinkbridge | grep "inet {}"'
                ''.format(config['static-linux-bridge-address'])
            ),
            errmsg=(
                'Bridge uplinkbridge does not have address {} .'
                ''.format(config['static-linux-bridge-address'])
            )
        )

        self._check_call(
            command=(
                'ip -4 -o route | grep "{} via {} dev uplinkbridge"'
                ''.format(
                    config['static-forwarding-network'],
                    netaddr.IPNetwork(
                        config['static-provider-router-address']
                    ).ip,
                )
            ),
            errmsg=(
                'Bridge uplinkbridge does not have route to forward '
                '{} via {} .'
                ''.format(
                    config['static-forwarding-network'],
                    netaddr.IPNetwork(
                        config['static-provider-router-address']
                    ).ip,
                )
            ),
        )

        self._check_output(
            command='sysctl -b net.ipv4.ip_forward',
            output='1',
            errmsg="IPv4 forwarding is not enabled.",
        )

        if config['static-outgoing-masquerade']:
            self._check_call(
                command=(
                    'iptables -t nat -C POSTROUTING -o {} -s {} -j MASQUERADE'
                    ' -m comment --comment JUJU-CHARM-MIDONET-GATEWAY-MANAGED'
                    ''.format(
                        config['static-outgoing-interface-name'],
                        config['static-forwarding-network'],
                    )
                ),
                errmsg=(
                    'Could not find iptables masquarading rule.'
                ),
            )
        else:
            self._check_call_negative(
                command=(
                    'iptables -t nat -C POSTROUTING -o {} -s {} -j MASQUERADE'
                    ' -m comment --comment JUJU-CHARM-MIDONET-GATEWAY-MANAGED'
                    ''.format(
                        config['static-outgoing-interface-name'],
                        config['static-forwarding-network'],
                    )
                ),
                errmsg=(
                    'Found iptables masquarading rule when it should not.'
                ),
            )

        self._check_call(
            command=(
                'iptables -C FORWARD'
                ' -s {} -j ACCEPT'
                ' -m comment --comment JUJU-CHARM-MIDONET-GATEWAY-MANAGED'
                ''.format(config['static-forwarding-network'])
            ),
            errmsg=(
                'Could not find iptables forwarding rule.'
            )
        )

        self.amulet_utils.log.info('Host configuration verified.')

    def _test_neutron_configuration(self, _):
        self.amulet_utils.log.info('Verifying Neutron configuration ...')

        keystone_address = self.keystone_sentry.relation(
            'identity-admin', 'midonet-gateway:identity-admin',
        )['private-address']

        self._check_call(
            command=(
                'neutron --os-auth-url http://{}:5000/v2.0'
                ' --os-tenant-name Admin --os-username admin'
                ' --os-password password net-show ext-net'.format(
                    keystone_address,
                )
            ),
            errmsg='Neutron network ext-net does not exists.'
        )

        self.amulet_utils.log.info('Neutron configuration verified.')

    def _test_midonet_configuration(self, config):
        self.amulet_utils.log.info('Verifying MidoNet configuration ...')

        midonet_api_address = self.midonet_api_sentry.relation(
            'midonet-api', 'midonet-gateway:midonet-api',
        )['private-address']

        self._check_call(
            command=(
                'echo -e "[cli]\napi_url = http://{}:8080/midonet-api\n'
                'username = admin\npassword = password\nproject_id = Admin"'
                ' > ~/.midonetrc'.format(midonet_api_address)
            ),
            errmsg='Could not write .midonetrc .'
        )

        provider_router = self._check_midonet_cli(
            command='router list name "MidoNet Provider Router" state up',
            errmsg='Could not find "MidoNet Provider Router".',
        )

        virtual_port_for_uplink = self._check_midonet_cli(
            command=(
                'router {} port list plugged yes address {} net {}'
                ''.format(
                    provider_router,
                    netaddr.IPNetwork(
                        config['static-provider-router-address']
                    ).ip,
                    netaddr.IPNetwork(
                        config['static-provider-router-address']
                    ).cidr,
                )
            ),
            errmsg=(
                'Could not find virtual port for uplink.',
            ),
        )

        self._check_midonet_cli(
            command=(
                'router {} route list '
                'type normal src 0.0.0.0/0 dst 0.0.0.0/0 gw {} port {}'
                ''.format(
                    provider_router,
                    netaddr.IPNetwork(
                        config['static-linux-bridge-address']
                    ).ip,
                    virtual_port_for_uplink,
                )
            ),
            errmsg=(
                'Could not find route for uplink.',
            ),
        )

        gateway_node = self._check_call(
            command=(
                'cat /etc/midonet_host_id.properties'
                '| grep host_uuid | cut -d "=" -f 2'
            ),
            errmsg=(
                'Could not read or parse /etc/midonet_host_id.properties .'
            ),
        )

        self._check_midonet_cli(
            command=(
                'host {} binding list interface veth1 port {}'
                ''.format(gateway_node, virtual_port_for_uplink)
            ),
            errmsg=(
                'Could not find binding for virtual port for uplink.'
            )
        )

        self.amulet_utils.log.info('MidoNet configuration verified.')
