from __future__ import absolute_import
from __future__ import division
from __future__ import print_function
from __future__ import unicode_literals

import os
import subprocess

import charmhelpers.core.templating
import netaddr
import netifaces

import lib.augeas
import lib.log
import lib.misc


def _bring_up_interface_if_not(interface):
    try:
        subprocess.check_call(['ifquery', '--state', interface])
    except subprocess.CalledProcessError:
        subprocess.check_call(['ifup', interface])
        lib.log.debug(
            lib.log.HOST_IFUP.format(interface),
        )


def _bring_down_interface_if_not(interface):
    try:
        subprocess.check_call(['ifquery', '--state', interface])
        subprocess.check_call(['ifdown', interface])
        lib.log.debug(
            lib.log.HOST_IFDOWN.format(interface),
        )
    except subprocess.CalledProcessError:
        pass


def _render_veth_interface_cfg(this_interface, that_interface):
    charmhelpers.core.templating.render(
        source='etc/network/interfaces.d/veth.cfg',
        target='/etc/network/interfaces.d/{}.cfg'.format(this_interface),
        context={
            'this_interface': this_interface,
            'that_interface': that_interface,
        },
    )


def _remove_veth_interface_cfg(this_interface):
    try:
        os.remove('/etc/network/interfaces.d/{}.cfg'.format(this_interface))
    except OSError:
        pass


def static_fix_interfaces_include():
    lib.log.debug(lib.log.HOST_FIX_INTERFACES_INCLUDE_START)

    aug = lib.augeas.Augeas()

    configured = False

    for source in aug.match('/files/etc/network/interfaces/source'):
        if aug.get(source) == '/etc/network/interfaces.d/*.cfg':
            configured = True
            lib.log.debug(lib.log.HOST_FIX_INTERFACES_INCLUDE_SKIP)

    if not configured:
        aug.set(
            '/files/etc/network/interfaces/source[last()+1]',
            '/etc/network/interfaces.d/*.cfg',
        )
        aug.save()
        lib.log.debug(lib.log.HOST_FIX_INTERFACES_INCLUDE_FIXED)


def static_setup_veth_pair():
    lib.log.debug(
        lib.log.HOST_SETUP_VETH_PAIR_START.format(
            lib.misc.STATIC_BRIDGE_IF_NAME,
            lib.misc.STATIC_PROVIDER_ROUTER_IF_NAME,
        )
    )

    _render_veth_interface_cfg(
        lib.misc.STATIC_BRIDGE_IF_NAME,
        lib.misc.STATIC_PROVIDER_ROUTER_IF_NAME,
    )
    _render_veth_interface_cfg(
        lib.misc.STATIC_PROVIDER_ROUTER_IF_NAME,
        lib.misc.STATIC_BRIDGE_IF_NAME,
    )
    _bring_up_interface_if_not(lib.misc.STATIC_BRIDGE_IF_NAME)
    _bring_up_interface_if_not(lib.misc.STATIC_PROVIDER_ROUTER_IF_NAME)


def static_teardown_veth_pair():
    lib.log.debug(
        lib.log.HOST_SETUP_VETH_PAIR_TEARDOWN.format(
            lib.misc.STATIC_BRIDGE_IF_NAME,
            lib.misc.STATIC_PROVIDER_ROUTER_IF_NAME,
        )
    )

    _bring_down_interface_if_not(lib.misc.STATIC_PROVIDER_ROUTER_IF_NAME)
    _bring_down_interface_if_not(lib.misc.STATIC_BRIDGE_IF_NAME)
    _remove_veth_interface_cfg(lib.misc.STATIC_PROVIDER_ROUTER_IF_NAME)
    _remove_veth_interface_cfg(lib.misc.STATIC_BRIDGE_IF_NAME)


def static_setup_bridge(rd_config):
    lib.log.debug(
        lib.log.HOST_SETUP_STATIC_BRIDGE_START,
    )

    bridge_address = netaddr.IPNetwork(
        rd_config['static-linux-bridge-address'],
    )
    provider_router_address = netaddr.IPNetwork(
        rd_config['static-provider-router-address'],
    )
    floating_ip_network = netaddr.IPNetwork(
        rd_config['static-forwarding-network'],
    )

    charmhelpers.core.templating.render(
        source='etc/network/interfaces.d/bridge.cfg',
        target='/etc/network/interfaces.d/{}.cfg'.format(
            lib.misc.STATIC_BRIDGE_NAME,
        ),
        context={
            'bridge_name': lib.misc.STATIC_BRIDGE_NAME,
            'bridge_if_name': lib.misc.STATIC_BRIDGE_IF_NAME,
            'bridge_address': str(bridge_address.ip),
            'bridge_netmask': str(bridge_address.netmask),
            'provider_router_address': str(provider_router_address.ip),
            'floating_ip_network': str(floating_ip_network),
        },
    )

    _bring_up_interface_if_not(lib.misc.STATIC_BRIDGE_NAME)


def static_teardown_bridge():
    lib.log.debug(
        lib.log.HOST_SETUP_STATIC_BRIDGE_TEARDOWN,
    )

    _bring_down_interface_if_not(lib.misc.STATIC_BRIDGE_NAME)

    try:
        os.remove('/etc/network/interfaces.d/{}.cfg'.format(
            lib.misc.STATIC_BRIDGE_NAME,
        ))
    except OSError:
        pass


def sysctl(key, desired_value):
    current_value = subprocess.check_output(['sysctl', '-b', str(key)])

    if str(current_value) != str(desired_value):
        subprocess.check_call(
            ['sysctl', '-w', '='.join((str(key), str(desired_value)))]
        )


def setup_netfilter_masquerade(rd_config):
    return_code = subprocess.call([
        'iptables',
        '-t', 'nat',
        '-C', 'POSTROUTING',
        '-o', str(rd_config['static-outgoing-interface-name']),
        '-s', str(rd_config['static-forwarding-network']),
        '-j', 'MASQUERADE',
        '-m', 'comment',
        '--comment', 'JUJU-CHARM-MIDONET-GATEWAY-MANAGED',
    ])

    if return_code == 0:
        lib.log.debug(lib.log.HOST_SETUP_MASQUERADING_NAT_RULE_FOUND)
    else:
        subprocess.check_call([
            'iptables',
            '-t', 'nat',
            '-I', 'POSTROUTING',
            '-o', str(rd_config['static-outgoing-interface-name']),
            '-s', str(rd_config['static-forwarding-network']),
            '-j', 'MASQUERADE',
            '-m', 'comment',
            '--comment', 'JUJU-CHARM-MIDONET-GATEWAY-MANAGED',
        ])
        lib.log.debug(lib.log.HOST_SETUP_MASQUERADING_NAT_RULE_CREATED)


def teardown_netfilter_masquerade(rd_config):
    return_code = subprocess.call([
        'iptables',
        '-t', 'nat',
        '-D', 'POSTROUTING',
        '-o', str(rd_config.previous('static-outgoing-interface-name')),
        '-s', str(rd_config.previous('static-forwarding-network')),
        '-j', 'MASQUERADE',
        '-m', 'comment',
        '--comment', 'JUJU-CHARM-MIDONET-GATEWAY-MANAGED',
    ])

    if return_code == 0:
        lib.log.debug(lib.log.HOST_SETUP_MASQUERADING_NAT_RULE_DELETED)
    else:
        lib.log.debug(lib.log.HOST_SETUP_MASQUERADING_NAT_RULE_NOT_DELETED)


def setup_netfilter_forward(rd_config):
    return_code = subprocess.call([
        'iptables',
        '-C', 'FORWARD',
        '-s', str(rd_config['static-forwarding-network']),
        '-j', 'ACCEPT',
        '-m', 'comment',
        '--comment', 'JUJU-CHARM-MIDONET-GATEWAY-MANAGED',
    ])

    if return_code == 0:
        lib.log.debug(lib.log.HOST_SETUP_MASQUERADING_FORWARD_RULE_FOUND)
    else:
        subprocess.check_call([
            'iptables',
            '-I', 'FORWARD',
            '-s', str(rd_config['static-forwarding-network']),
            '-j', 'ACCEPT',
            '-m', 'comment',
            '--comment', 'JUJU-CHARM-MIDONET-GATEWAY-MANAGED',
        ])
        lib.log.debug(lib.log.HOST_SETUP_MASQUERADING_FORWARD_RULE_CREATED)


def teardown_netfilter_forward(rd_config):
    return_code = subprocess.call([
        'iptables',
        '-D', 'FORWARD',
        '-s', str(rd_config.previous('static-forwarding-network')),
        '-j', 'ACCEPT',
        '-m', 'comment',
        '--comment', 'JUJU-CHARM-MIDONET-GATEWAY-MANAGED',
    ])

    if return_code == 0:
        lib.log.debug(lib.log.HOST_SETUP_MASQUERADING_FORWARD_RULE_DELETED)
    else:
        lib.log.debug(lib.log.HOST_SETUP_MASQUERADING_FORWARD_RULE_NOT_DELETED)


def ifname_from_macaddr(mac_address):
    interface_name = None

    for interface in netifaces.interfaces():  # pylint: disable=no-member
        try:
            for ifaddr in netifaces.ifaddresses(interface)[netifaces.AF_LINK]:  # noqa pylint: disable=no-member
                if netaddr.EUI(mac_address) == netaddr.EUI(ifaddr['addr']):
                    interface_name = interface
        except KeyError:
            pass

    return interface_name
