from __future__ import absolute_import
from __future__ import division
from __future__ import print_function
from __future__ import unicode_literals

import netaddr

import lib.log
import lib.misc


@lib.misc.retry_midonet_exception_result
def get_provider_router(midonet_client):
    provider_router = None

    for router in midonet_client.get_routers({}):
        if router.get_name() == 'MidoNet Provider Router':
            provider_router = router

    if provider_router:
        lib.log.debug(
            lib.log.MIDONET_PROVIDER_ROUTER_FOUND.format(
                provider_router.get_id(),
            ),
        )
    else:
        lib.log.debug(
            lib.log.MIDONET_PROVIDER_ROUTER_NOT_FOUND,
        )

    return provider_router


@lib.misc.retry_midonet_exception
def router_get_port(router, port_address):
    port_address = netaddr.IPNetwork(port_address)
    port_object = None

    for port in router.get_ports():
        _port_address = netaddr.IPAddress(port.get_port_address())
        _port_address_prefixlen = port.get_network_length()

        conditions = []
        conditions.append(_port_address == port_address.ip)
        conditions.append(_port_address_prefixlen == port_address.prefixlen)

        if all(conditions):
            port_object = port

    if port_object:
        lib.log.debug(
            lib.log.MIDONET_ROUTER_PORT_FOUND.format(
                port_address.ip,
                port_address.prefixlen,
                port_object.get_id(),
                router.get_id(),
            )
        )
    else:
        lib.log.debug(
            lib.log.MIDONET_ROUTER_PORT_NOT_FOUND.format(
                port_address.ip,
                port_address.prefixlen,
                router.get_id(),
            )
        )

    return port_object


@lib.misc.retry_midonet_exception
def router_add_port(router, port_address):
    port_address = netaddr.IPNetwork(port_address)
    port_object = router_get_port(
        router,
        port_address,
    )

    if not port_object:
        port_object = (
            router
            .add_port()
            .port_address(str(port_address.ip))
            .network_address(str(port_address.network))
            .network_length(int(port_address.prefixlen))
            .create()
        )
        lib.log.debug(
            lib.log.MIDONET_ROUTER_PORT_CREATED.format(
                port_address.ip,
                port_address.prefixlen,
                port_object.get_id(),
                router.get_id(),
            )
        )

    return port_object


@lib.misc.retry_midonet_exception
def port_delete(port):
    port_address = port.get_port_address()
    network_length = port.get_network_length()
    port_id = port.get_id()
    router_id = port.get_device_id()

    port.delete()

    lib.log.debug(
        lib.log.MIDONET_PORT_DELETED.format(
            port_address,
            network_length,
            port_id,
            router_id,
        )
    )


@lib.misc.retry_midonet_exception
def port_get_bgp(port, local_as, peer_address, peer_as):
    peer_address = netaddr.IPAddress(peer_address)
    bgp_object = None

    for bgp in port.get_bgps():
        _local_as = bgp.get_local_as()
        _peer_address = netaddr.IPAddress(bgp.get_peer_addr())
        _peer_as = bgp.get_peer_as()

        conditions = []
        conditions.append(_local_as == local_as)
        conditions.append(_peer_address == peer_address)
        conditions.append(_peer_as == peer_as)

        if all(conditions):
            bgp_object = bgp

    if bgp_object:
        lib.log.debug(
            lib.log.MIDONET_PORT_BGP_FOUND.format(
                local_as,
                peer_address,
                peer_as,
                bgp_object.get_id(),
                port.get_id(),
            ),
        )
    else:
        lib.log.debug(
            lib.log.MIDONET_PORT_BGP_NOT_FOUND.format(
                local_as,
                peer_address,
                peer_as,
                port.get_id(),
            ),
        )

    return bgp_object


@lib.misc.retry_midonet_exception
def port_add_bgp(port, local_as, peer_address, peer_as):
    peer_address = netaddr.IPAddress(peer_address)
    bgp_object = port_get_bgp(
        port,
        local_as,
        peer_address,
        peer_as,
    )

    if not bgp_object:
        bgp_object = (
            port
            .add_bgp()
            .local_as(int(local_as))
            .peer_addr(str(peer_address))
            .peer_as(int(peer_as))
            .create()
        )
        lib.log.debug(
            lib.log.MIDONET_PORT_BGP_CREATED.format(
                local_as,
                peer_address,
                peer_as,
                bgp_object.get_id(),
                port.get_id(),
            ),
        )

    return bgp_object


@lib.misc.retry_midonet_exception
def bgp_get_ad_route(bgp, network):
    network = netaddr.IPNetwork(network).cidr
    ad_route_object = None

    for ad_route in bgp.get_ad_routes():
        _network = netaddr.IPNetwork('{}/{}'.format(
            ad_route.get_nw_prefix(),
            ad_route.get_prefix_length(),
        )).cidr

        if _network == network:
            ad_route_object = ad_route

    if ad_route_object:
        lib.log.debug(
            lib.log.MIDONET_BGP_AD_ROUTE_FOUND.format(
                network,
                ad_route_object.get_id(),
                bgp.get_id(),
            )
        )
    else:
        lib.log.debug(
            lib.log.MIDONET_BGP_AD_ROUTE_NOT_FOUND.format(
                network,
                bgp.get_id(),
            )
        )

    return ad_route_object


@lib.misc.retry_midonet_exception
def bgp_add_ad_route(bgp, network):
    network = netaddr.IPNetwork(network).cidr
    ad_route_object = bgp_get_ad_route(
        bgp,
        network
    )

    if not ad_route_object:
        ad_route_object = (
            bgp
            .add_ad_route()
            .nw_prefix(str(network.network))
            .nw_prefix_length(int(network.prefixlen))
            .create()
        )
        lib.log.debug(
            lib.log.MIDONET_BGP_AD_ROUTE_CREATED.format(
                network,
                ad_route_object.get_id(),
                bgp.get_id(),
            )
        )

    return ad_route_object


@lib.misc.retry_midonet_exception
def router_get_route(router,  # pylint: disable=too-many-arguments
                     src_net,
                     dst_net,
                     next_hop_port,
                     next_hop_gateway=None,
                     route_type=None):
    src_net = netaddr.IPNetwork(src_net).cidr
    dst_net = netaddr.IPNetwork(dst_net).cidr

    if next_hop_gateway:
        try:
            next_hop_gateway = str(netaddr.IPAddress(next_hop_gateway))
        except ValueError:
            next_hop_gateway = str(netaddr.IPNetwork(next_hop_gateway).ip)

    if route_type not in lib.misc.ROUTE_TYPES:
        raise ValueError

    route_object = None

    for route in router.get_routes():
        _src_net = netaddr.IPNetwork('{}/{}'.format(
            route.get_src_network_addr(),
            route.get_src_network_length(),
        )).cidr
        _dst_net = netaddr.IPNetwork('{}/{}'.format(
            route.get_dst_network_addr(),
            route.get_dst_network_length(),
        )).cidr
        _next_hop_port = route.get_next_hop_port()

        if route.get_next_hop_gateway():
            if route.get_next_hop_gateway() == 'None':
                _next_hop_gateway = None
            else:
                _next_hop_gateway = route.get_next_hop_gateway()
        else:
            _next_hop_gateway = None

        _route_type = route.get_type()

        comparison_list = []
        comparison_list.append(_src_net == src_net)
        comparison_list.append(_dst_net == dst_net)
        comparison_list.append(_next_hop_port == next_hop_port.get_id())
        comparison_list.append(_next_hop_gateway == next_hop_gateway)
        comparison_list.append(_route_type == route_type)

        if all(comparison_list):
            route_object = route

    if route_object:
        lib.log.debug(
            lib.log.MIDONET_ROUTER_ROUTE_FOUND.format(
                src_net,
                dst_net,
                next_hop_port.get_id(),
                next_hop_gateway,
                route_type,
                route_object.get_id(),
                router.get_id(),
            ),
        )
    else:
        lib.log.debug(
            lib.log.MIDONET_ROUTER_ROUTE_NOT_FOUND.format(
                src_net,
                dst_net,
                next_hop_port.get_id(),
                next_hop_gateway,
                route_type,
                router.get_id(),
            ),
        )

    return route_object


@lib.misc.retry_midonet_exception
def router_add_route(router,  # pylint: disable=too-many-arguments
                     src_net,
                     dst_net,
                     next_hop_port,
                     next_hop_gateway=None,
                     route_type=None):
    src_net = netaddr.IPNetwork(src_net).cidr
    dst_net = netaddr.IPNetwork(dst_net).cidr

    if next_hop_gateway:
        try:
            next_hop_gateway = str(netaddr.IPAddress(next_hop_gateway))
        except ValueError:
            next_hop_gateway = str(netaddr.IPNetwork(next_hop_gateway).ip)

    if route_type not in lib.misc.ROUTE_TYPES:
        raise ValueError

    route_object = router_get_route(
        router,
        src_net,
        dst_net,
        next_hop_port,
        next_hop_gateway,
        route_type,
    )

    if not route_object:
        route_object = (
            router
            .add_route()
            .src_network_addr(str(src_net.network))
            .src_network_length(int(src_net.prefixlen))
            .dst_network_addr(str(dst_net.network))
            .dst_network_length(int(dst_net.prefixlen))
            .next_hop_port(str(next_hop_port.get_id()))
            .next_hop_gateway(next_hop_gateway)
            .type(route_type)
            .create()
        )
        lib.log.debug(
            lib.log.MIDONET_ROUTER_ROUTE_CREATED.format(
                src_net,
                dst_net,
                next_hop_port.get_id(),
                next_hop_gateway,
                route_type,
                route_object.get_id(),
                router.get_id(),
            ),
        )

    return route_object


@lib.misc.retry_midonet_exception
def route_delete(route):
    src_net = netaddr.IPNetwork('{}/{}'.format(
        route.get_src_network_addr(),
        route.get_src_network_length(),
    )).cidr
    dst_net = netaddr.IPNetwork('{}/{}'.format(
        route.get_dst_network_addr(),
        route.get_dst_network_length(),
    )).cidr
    next_hop_port = route.get_next_hop_port()
    next_hop_gateway = route.get_next_hop_gateway()
    route_type = route.get_type()
    route_id = route.get_id()
    router_id = route.get_router_id()

    route.delete()

    lib.log.debug(
        lib.log.MIDONET_ROUTE_DELETED.format(
            str(src_net),
            str(dst_net),
            next_hop_port,
            next_hop_gateway,
            route_type,
            route_id,
            router_id,
        )
    )


@lib.misc.retry_midonet_exception_result
def get_gateway_node(midonet_client, host_uuid):
    gateway_node = None

    for host in midonet_client.get_hosts():
        if host.get_id() == str(host_uuid):
            gateway_node = host

    if gateway_node:
        lib.log.debug(
            lib.log.MIDONET_HOST_FOUND.format(
                host_uuid,
                gateway_node.get_name(),
            ),
        )
    else:
        lib.log.debug(
            lib.log.MIDONET_HOST_NOT_FOUND.format(
                host_uuid,
            ),
        )

    return gateway_node


@lib.misc.retry_midonet_exception_result
def wait_for_node_to_be_in_tzone(midonet_client, node):
    tunnel_zone_gateway_node = None

    for tunnel_zone in midonet_client.get_tunnel_zones():
        for tunnel_zone_host in tunnel_zone.get_hosts():
            if tunnel_zone_host.get_host_id() == node.get_id():
                tunnel_zone_gateway_node = tunnel_zone_host

    if tunnel_zone_gateway_node:
        lib.log.debug(
            lib.log.MIDONET_HOST_IN_A_TZONE.format(
                node.get_id(),
                tunnel_zone_gateway_node.get_tunnel_zone_id(),
            ),
        )
    else:
        lib.log.debug(
            lib.log.MIDONET_HOST_NOT_IN_ANY_TZONE.format(
                node.get_id(),
            ),
        )

    return tunnel_zone_gateway_node


@lib.misc.retry_midonet_exception
def host_get_binding(host, port, interface):
    host_interface_port_object = None

    for host_port in host.get_ports():
        _port_uuid = host_port.get_port_id()
        _interface = host_port.get_interface_name()

        comparison_list = []
        comparison_list.append(_port_uuid == port.get_id())
        comparison_list.append(_interface == interface)

        if all(comparison_list):
            host_interface_port_object = host_port

    if host_interface_port_object:
        lib.log.debug(
            lib.log.MIDONET_HOST_INTERFACE_PORT_FOUND.format(
                port.get_id(),
                interface,
            ),
        )
    else:
        lib.log.debug(
            lib.log.MIDONET_HOST_INTERFACE_PORT_NOT_FOUND.format(
                port.get_id(),
                interface,
            ),
        )

    return host_interface_port_object


@lib.misc.retry_midonet_exception
def host_add_binding(host, port, interface):
    host_interface_port_object = host_get_binding(
        host,
        port,
        interface,
    )

    if not host_interface_port_object:
        host_interface_port_object = (
            host
            .add_host_interface_port()
            .port_id(port.get_id())
            .interface_name(interface)
            .create()
        )
        lib.log.debug(
            lib.log.MIDONET_HOST_INTERFACE_PORT_CREATED.format(
                port.get_id(),
                interface,
                host.get_id(),
            ),
        )

    return host_interface_port_object


@lib.misc.retry_midonet_exception
def get_stateful_port_group(client, name):
    port_group_object = None

    for port_group in client.get_port_groups({}):
        _name = port_group.get_name()
        _stateful = port_group.is_stateful()

        comparison_list = []
        comparison_list.append(_name == name)
        comparison_list.append(_stateful is True)

        if all(comparison_list):
            port_group_object = port_group

    if port_group_object:
        lib.log.debug(
            lib.log.MIDONET_PORT_GROUP_FOUND.format(
                name,
                port_group_object.get_id(),
            ),
        )
    else:
        lib.log.debug(
            lib.log.MIDONET_PORT_GROUP_NOT_FOUND.format(
                name,
            ),
        )

    return port_group_object


@lib.misc.retry_midonet_exception
def create_stateful_port_group(client, name, tenant_id):
    port_group_object = get_stateful_port_group(
        client,
        name,
    )

    if not port_group_object:
        port_group_object = (
            client
            .add_port_group()
            .name(name)
            .stateful(True)
            .tenant_id(tenant_id)
            .create()
        )
        lib.log.debug(
            lib.log.MIDONET_PORT_GROUP_CREATED.format(
                name,
                port_group_object.get_id(),
            ),
        )

    return port_group_object


@lib.misc.retry_midonet_exception
def port_group_get_port(port_group, port):
    port_group_port_object = None

    for port_group_port in port_group.get_ports():
        if port_group_port.get_port_id() == port.get_id():
            port_group_port_object = port_group_port

    if port_group_port_object:
        lib.log.debug(
            lib.log.MIDONET_PORT_GROUP_PORT_FOUND.format(
                port.get_id(),
                port_group.get_id(),
            ),
        )
    else:
        lib.log.debug(
            lib.log.MIDONET_PORT_GROUP_PORT_NOT_FOUND.format(
                port.get_id(),
                port_group.get_id(),
            ),
        )

    return port_group_port_object


@lib.misc.retry_midonet_exception
def port_group_add_port(port_group, port):
    port_group_port_object = port_group_get_port(
        port_group,
        port,
    )

    if not port_group_port_object:
        port_group_port_object = (
            port_group
            .add_port_group_port()
            .port_id(port.get_id())
            .create()
        )
        lib.log.debug(
            lib.log.MIDONET_PORT_GROUP_PORT_CREATED.format(
                port.get_id(),
                port_group.get_id(),
            ),
        )

    return port_group_port_object
