from __future__ import absolute_import
from __future__ import division
from __future__ import print_function
from __future__ import unicode_literals

import yaml

import lib.log
import lib.midonet_api
import lib.misc


def setup_static_midonet_gateway(midonet_client, rd_config):
    lib.log.debug(lib.log.MIDONET_SETUP_STARTED_STATIC)

    provider_router_if_name = lib.misc.STATIC_PROVIDER_ROUTER_IF_NAME
    provider_router_address = rd_config['static-provider-router-address']
    linux_bridge_address = rd_config['static-linux-bridge-address']

    provider_router = lib.midonet_api.get_provider_router(
        midonet_client=midonet_client,
    )

    virtual_port_for_uplink = lib.midonet_api.router_add_port(
        router=provider_router,
        port_address=provider_router_address,
    )

    lib.midonet_api.router_add_route(
        router=provider_router,
        src_net='0.0.0.0/0',
        dst_net='0.0.0.0/0',
        next_hop_port=virtual_port_for_uplink,
        next_hop_gateway=linux_bridge_address,
        route_type='Normal',
    )

    gateway_node = lib.midonet_api.get_gateway_node(
        midonet_client=midonet_client,
        host_uuid=lib.misc.get_host_uuid(),
    )

    lib.midonet_api.wait_for_node_to_be_in_tzone(
        midonet_client=midonet_client,
        node=gateway_node,
    )

    lib.midonet_api.host_add_binding(
        gateway_node,
        port=virtual_port_for_uplink,
        interface=provider_router_if_name,
    )


def setup_bgp_midonet_gateway(keystone_client, midonet_client, rd_config):
    lib.log.debug(lib.log.MIDONET_SETUP_STARTED_BGP)

    bgp_advertised_networks = yaml.load(rd_config['bgp-advertised-networks'])
    bgp_topology = yaml.load(rd_config['bgp-topology'])

    provider_router = lib.midonet_api.get_provider_router(
        midonet_client=midonet_client,
    )

    this_host = lib.midonet_api.get_gateway_node(
        midonet_client=midonet_client,
        host_uuid=lib.misc.get_host_uuid(),
    )

    lib.midonet_api.wait_for_node_to_be_in_tzone(
        midonet_client=midonet_client,
        node=this_host,
    )

    stateful_port_group = lib.midonet_api.create_stateful_port_group(
        client=midonet_client,
        name=rd_config['port-group-name'],
        tenant_id=keystone_client.tenant_id,
    )

    for virtual_port_config in bgp_topology:
        virtual_port_mac_addr = virtual_port_config['local_mac_address']

        if not lib.host.ifname_from_macaddr(virtual_port_mac_addr):
            continue

        virtual_port = lib.midonet_api.router_add_port(
            router=provider_router,
            port_address=virtual_port_config['local_ip_address'],
        )

        lib.midonet_api.router_add_route(
            router=provider_router,
            src_net='0.0.0.0/0',
            dst_net=virtual_port_config['local_ip_address'],
            next_hop_port=virtual_port,
            route_type='Normal',
        )

        lib.midonet_api.host_add_binding(
            host=this_host,
            port=virtual_port,
            interface=lib.host.ifname_from_macaddr(virtual_port_mac_addr),
        )

        lib.midonet_api.port_group_add_port(
            port_group=stateful_port_group,
            port=virtual_port,
        )

        for peer_ip_address in virtual_port_config['peer_ip_addresses']:
            bgp = lib.midonet_api.port_add_bgp(
                port=virtual_port,
                local_as=rd_config['bgp-local-as-number'],
                peer_as=rd_config['bgp-peer-as-number'],
                peer_address=peer_ip_address,
            )

            for bgp_advertised_network in bgp_advertised_networks:
                lib.midonet_api.bgp_add_ad_route(
                    bgp,
                    bgp_advertised_network,
                )


def cleanup_static_midonet_gateway(midonet_client, rd_config):
    router = lib.midonet_api.get_provider_router(midonet_client)

    if router:
        port = lib.midonet_api.router_get_port(
            router=router,
            port_address=rd_config.previous('static-provider-router-address'),
        )

        if port:
            lib.midonet_api.port_delete(port)


def cleanup_bgp_midonet_gateway(midonet_client, rd_config):
    bgp_topology = yaml.load(rd_config.previous('bgp-topology'))

    router = lib.midonet_api.get_provider_router(midonet_client)

    if router:
        for virtual_port_config in bgp_topology:
            port = lib.midonet_api.router_get_port(
                router=router,
                port_address=virtual_port_config['local_ip_address'],
            )

            if port:
                lib.midonet_api.port_delete(port)
