from __future__ import absolute_import
from __future__ import division
from __future__ import print_function
from __future__ import unicode_literals

import functools
import socket

import httplib
import keystoneclient.openstack.common.apiclient.exceptions as kocae
import keystoneclient.v2_0.client
import midonetclient.api
import midonetclient.exc
import neutronclient.common.exceptions  # pylint: disable=wrong-import-order
import neutronclient.v2_0.client  # pylint: disable=wrong-import-order
import retrying  # pylint: disable=wrong-import-order
import webob.exc  # pylint: disable=wrong-import-order

import lib.jprops

STOP_MAX_DELAY = 600*1000
WAIT_FIXED = 1000
ROUTE_TYPES = ['Normal', 'BlackHole', 'Reject', 'Local']

STATIC_BRIDGE_NAME = 'uplinkbridge'
STATIC_BRIDGE_IF_NAME = 'veth0'
STATIC_PROVIDER_ROUTER_IF_NAME = 'veth1'


def _retry_if_keystone_client_error(exception):
    conditions = []
    conditions.append(isinstance(
        exception,
        kocae.AuthorizationFailure,
    ))

    return any(conditions)


def _retry_if_neutron_client_error(exception):
    conditions = []
    conditions.append(isinstance(
        exception,
        neutronclient.common.exceptions.ConnectionFailed,
    ))
    conditions.append(isinstance(
        exception,
        neutronclient.common.exceptions.InternalServerError,
    ))
    conditions.append(isinstance(
        exception,
        neutronclient.common.exceptions.NotFound
    ))
    conditions.append(isinstance(
        exception,
        neutronclient.common.exceptions.Unauthorized,
    ))

    return any(conditions)


def _retry_if_midonet_client_error(exception):
    conditions = []
    conditions.append(isinstance(
        exception,
        httplib.HTTPException,
    ))
    conditions.append(isinstance(
        exception,
        midonetclient.exc.MidoApiConnectionError,
    ))
    conditions.append(isinstance(
        exception,
        socket.error,
    ))
    conditions.append(isinstance(
        exception,
        webob.exc.HTTPException,
    ))

    return any(conditions)


def _retry_if_result_is_none(result):
    return result is None


def retry_keystone_exception(function):
    @functools.wraps(function)
    def wrapped_function(*args, **kwargs):
        return retrying.Retrying(
            stop_max_delay=STOP_MAX_DELAY,
            wait_fixed=WAIT_FIXED,
            retry_on_exception=_retry_if_keystone_client_error,
        ).call(function, *args, **kwargs)

    return wrapped_function


def retry_neutron_exception(function):
    @functools.wraps(function)
    def wrapped_function(*args, **kwargs):
        return retrying.Retrying(
            stop_max_delay=STOP_MAX_DELAY,
            wait_fixed=WAIT_FIXED,
            retry_on_exception=_retry_if_neutron_client_error,
        ).call(function, *args, **kwargs)

    return wrapped_function


def retry_neutron_exception_result(function):
    @functools.wraps(function)
    def wrapped_function(*args, **kwargs):
        return retrying.Retrying(
            stop_max_delay=STOP_MAX_DELAY,
            wait_fixed=WAIT_FIXED,
            retry_on_exception=_retry_if_neutron_client_error,
            retry_on_result=_retry_if_result_is_none,
        ).call(function, *args, **kwargs)

    return wrapped_function


def retry_midonet_exception(function):
    @functools.wraps(function)
    def wrapped_function(*args, **kwargs):
        return retrying.Retrying(
            stop_max_delay=STOP_MAX_DELAY,
            wait_fixed=WAIT_FIXED,
            retry_on_exception=_retry_if_midonet_client_error,
        ).call(function, *args, **kwargs)

    return wrapped_function


def retry_midonet_exception_result(function):
    @functools.wraps(function)
    def wrapped_function(*args, **kwargs):
        return retrying.Retrying(
            stop_max_delay=STOP_MAX_DELAY,
            wait_fixed=WAIT_FIXED,
            retry_on_exception=_retry_if_midonet_client_error,
            retry_on_result=_retry_if_result_is_none,
        ).call(function, *args, **kwargs)

    return wrapped_function


def get_host_uuid():
    try:
        with open('/etc/midonet_host_id.properties') as file_object:
            properties = lib.jprops.load_properties(file_object)
    except IOError:
        with open('/etc/midolman/host_uuid.properties') as file_object:
            properties = lib.jprops.load_properties(file_object)

    return properties['host_uuid']


@retry_keystone_exception
def get_keystoneclient(keystone_context):
    return keystoneclient.v2_0.client.Client(
        username=keystone_context['service_username'],
        password=keystone_context['service_password'],
        tenant_name=keystone_context['service_tenant_name'],
        auth_url='http://{}:{}/v2.0'.format(
            keystone_context['service_hostname'],
            keystone_context['service_port'],
        )
    )


@retry_neutron_exception
def get_neutronclient(keystone_context):
    return neutronclient.v2_0.client.Client(
        username=keystone_context['service_username'],
        password=keystone_context['service_password'],
        tenant_name=keystone_context['service_tenant_name'],
        auth_url='http://{}:{}/v2.0'.format(
            keystone_context['service_hostname'],
            keystone_context['service_port'],
        )
    )


@retry_midonet_exception
def get_midonetclient(keystone_context, midonet_context):
    return midonetclient.api.MidonetApi(
        base_uri='http://{}:{}/midonet-api'.format(
            midonet_context['host'],
            midonet_context['port'],
        ),
        username=keystone_context['service_username'],
        password=keystone_context['service_password'],
    )
