from __future__ import absolute_import
from __future__ import division
from __future__ import print_function
from __future__ import unicode_literals

import charmhelpers.core.hookenv
import charmhelpers.core.services.base as ccsb

import lib.config
import lib.host
import lib.midonet_api
import lib.midonet_conf
import lib.misc
import lib.neutron_api
import lib.neutron_conf
import lib.relations


class ServiceManagerCallbackBase(ccsb.ManagerCallback):
    def __call__(self, manager, service_name, _):
        self.setup_required_data(manager, service_name)
        self.setup_api_client_objects()

    def setup_required_data(self, manager, service_name):
        lib.log.debug(
            lib.log.SERVICE_MANAGER_START_CALLBACK_REQUIRED_DATA,
        )

        required_data = manager.get_service(service_name)['required_data']

        for required_datum in required_data:
            if 'config' in required_datum:
                self.rd_config = charmhelpers.core.hookenv.config()
            elif 'identity-admin' in required_datum:
                self.rd_identity_admin = required_datum['identity-admin'][0]
            elif 'midonet-api' in required_datum:
                self.rd_midonet_api = required_datum['midonet-api'][0]

    def setup_api_client_objects(self):
        lib.log.debug(
            lib.log.SERVICE_MANAGER_START_CALLBACK_API_CLIENTS,
        )

        self.keystoneclient = lib.misc.get_keystoneclient(
            self.rd_identity_admin,
        )

        self.neutronclient = lib.misc.get_neutronclient(
            self.rd_identity_admin,
        )

        self.midonetclient = lib.misc.get_midonetclient(
            self.rd_identity_admin,
            self.rd_midonet_api,
        )


class ServiceManagerStartCallback(ServiceManagerCallbackBase):
    def __call__(self, manager, service_name, _):
        lib.log.info_with_status(
            'maintenance',
            lib.log.SERVICE_MANAGER_START_CALLBACK_BEGIN,
        )

        _super = super(ServiceManagerStartCallback, self)
        _super.__call__(manager, service_name, _)

        teardown_functions = {}
        teardown_functions['static'] = self.teardown_static
        teardown_functions['bgp'] = self.teardown_bgp

        setup_functions = {}
        setup_functions['static'] = self.setup_static
        setup_functions['bgp'] = self.setup_bgp

        try:
            lib.config.check_config_consistency(self.rd_config)

            if lib.config.is_config_different(self.rd_config):
                previous_uplink_type = self.rd_config.previous('uplink-type')
                teardown_functions[previous_uplink_type]()

            uplink_type = self.rd_config['uplink-type']
            setup_functions[uplink_type]()

            lib.log.info_with_status(
                'active',
                lib.log.SERVICE_MANAGER_START_CALLBACK_FINISHED,
            )

        except lib.exceptions.ConsistencyError as exception:
            lib.log.error_with_status(
                'blocked',
                exception.status_message
            )

    def teardown_static(self):
        lib.log.info_with_status(
            'maintenance',
            lib.log.SERVICE_MANAGER_CALLBACK_TEARDOWN_STATIC,
        )

        lib.host.teardown_netfilter_forward(self.rd_config)

        if self.rd_config.previous('static-outgoing-masquerade'):
            lib.host.teardown_netfilter_masquerade(self.rd_config)

        lib.midonet_conf.cleanup_static_midonet_gateway(
            self.midonetclient,
            self.rd_config,
        )

        lib.host.static_teardown_bridge()

        lib.host.static_teardown_veth_pair()

    def teardown_bgp(self):
        lib.log.info_with_status(
            'maintenance',
            lib.log.SERVICE_MANAGER_CALLBACK_TEARDOWN_BGP,
        )

        lib.midonet_conf.cleanup_bgp_midonet_gateway(
            self.midonetclient,
            self.rd_config,
        )

    def setup_static(self):
        lib.log.info_with_status(
            'maintenance',
            lib.log.SERVICE_MANAGER_CALLBACK_SETUP_STATIC,
        )

        lib.host.static_fix_interfaces_include()

        lib.host.static_setup_veth_pair()

        lib.host.static_setup_bridge(self.rd_config)

        lib.host.sysctl('net.ipv4.ip_forward', '1')

        lib.neutron_conf.create_neutron_external_network(
            self.neutronclient,
            self.rd_config,
        )

        lib.midonet_conf.setup_static_midonet_gateway(
            self.midonetclient,
            self.rd_config,
        )

        if self.rd_config['static-outgoing-masquerade']:
            lib.host.setup_netfilter_masquerade(self.rd_config)

        lib.host.setup_netfilter_forward(self.rd_config)

    def setup_bgp(self):
        lib.log.info_with_status(
            'maintenance',
            lib.log.SERVICE_MANAGER_CALLBACK_SETUP_BGP,
        )

        lib.neutron_conf.create_neutron_external_network(
            self.neutronclient,
            self.rd_config,
        )

        lib.midonet_conf.setup_bgp_midonet_gateway(
            self.keystoneclient,
            self.midonetclient,
            self.rd_config,
        )


def main():
    lib.log.debug(
        'Invoking hook "{}"'.format(charmhelpers.core.hookenv.hook_name())
    )

    ccsb.ServiceManager([
        {
            'service': 'midonet-gateway',
            'required_data': [
                lib.config.MidonetGatewayRequiredConfig(),
                lib.relations.IdentityAdminRelation(),
                lib.relations.NeutronPluginApiRelation(),
                lib.relations.MidonetApiRelation(),
                lib.relations.MidonetHostRelation(),
            ],
            'start': ServiceManagerStartCallback(),
            'stop': [],
        },
    ]).manage()

    lib.log.debug(
        'Finished hook "{}"'.format(charmhelpers.core.hookenv.hook_name())
    )
