from __future__ import absolute_import
from __future__ import division
from __future__ import print_function
from __future__ import unicode_literals

import charmhelpers.core.hookenv
import httpretty
import mock

import lib.log
import lib.midonet_api
import lib.misc
import lib.tests.mock_midonet_api
import lib.tests.mocks.midonet


class TestRouterPort(object):
    def setup(self):
        self.patcher_log = mock.patch(
            'charmhelpers.core.hookenv.log',
        )
        self.mocked_log = self.patcher_log.start()

        httpretty.enable()
        httpretty.HTTPretty.allow_net_connect = False

        lib.tests.mock_midonet_api.mock_login()
        lib.tests.mock_midonet_api.mock_root()
        lib.tests.mock_midonet_api.mock_system_state()
        lib.tests.mock_midonet_api.mock_routers_list_pass()

        self.provider_router = lib.midonet_api.get_provider_router(
            midonet_client=lib.tests.mocks.midonet.get_bogus_midonetclient(),
        )

    def teardown(self):
        httpretty.disable()
        httpretty.reset()

        self.patcher_log.stop()

    def test_list_pass(self):
        self.mocked_log.reset_mock()
        lib.tests.mock_midonet_api.mock_router_port_list_pass()

        port = lib.midonet_api.router_get_port(
            router=self.provider_router,
            port_address=str(lib.tests.mock_midonet_api.IPNETWORK_PORT_PASS),
        )

        assert port.get_port_address() == str(
            lib.tests.mock_midonet_api.IPNETWORK_PORT_PASS.ip,
        )
        assert port.get_network_length() == int(
            lib.tests.mock_midonet_api.IPNETWORK_PORT_PASS.prefixlen,
        )
        assert port.get_id() == str(
            lib.tests.mocks.midonet.UUID_PORT_ID_PASS,
        )
        assert port.get_device_id() == str(
            lib.tests.mocks.midonet.UUID_ROUTER_ID_PASS,
        )
        self.mocked_log.assert_called_once_with(
            lib.log.MIDONET_ROUTER_PORT_FOUND.format(
                lib.tests.mock_midonet_api.IPNETWORK_PORT_PASS.ip,
                lib.tests.mock_midonet_api.IPNETWORK_PORT_PASS.prefixlen,
                lib.tests.mocks.midonet.UUID_PORT_ID_PASS,
                lib.tests.mocks.midonet.UUID_ROUTER_ID_PASS,
            ),
            charmhelpers.core.hookenv.DEBUG,
        )

    def test_list_fail(self):
        self.mocked_log.reset_mock()
        lib.tests.mock_midonet_api.mock_router_port_list_fail()

        port = lib.midonet_api.router_get_port(
            router=self.provider_router,
            port_address=str(lib.tests.mock_midonet_api.IPNETWORK_PORT_PASS),
        )

        assert port is None
        self.mocked_log.assert_called_once_with(
            lib.log.MIDONET_ROUTER_PORT_NOT_FOUND.format(
                lib.tests.mock_midonet_api.IPNETWORK_PORT_PASS.ip,
                lib.tests.mock_midonet_api.IPNETWORK_PORT_PASS.prefixlen,
                lib.tests.mocks.midonet.UUID_ROUTER_ID_PASS,
            ),
            charmhelpers.core.hookenv.DEBUG,
        )

    def test_add(self):
        self.mocked_log.reset_mock()
        lib.tests.mock_midonet_api.mock_router_port_list_fail()
        lib.tests.mock_midonet_api.mock_router_port_add()
        lib.tests.mock_midonet_api.mock_port_get()

        port = lib.midonet_api.router_add_port(
            router=self.provider_router,
            port_address=str(lib.tests.mock_midonet_api.IPNETWORK_PORT_PASS),
        )

        assert port.get_port_address() == str(
            lib.tests.mock_midonet_api.IPNETWORK_PORT_PASS.ip,
        )
        assert port.get_network_length() == int(
            lib.tests.mock_midonet_api.IPNETWORK_PORT_PASS.prefixlen,
        )
        assert port.get_id() == str(
            lib.tests.mocks.midonet.UUID_PORT_ID_PASS,
        )
        assert port.get_device_id() == str(
            lib.tests.mocks.midonet.UUID_ROUTER_ID_PASS,
        )
        self.mocked_log.assert_called_with(
            lib.log.MIDONET_ROUTER_PORT_CREATED.format(
                lib.tests.mock_midonet_api.IPNETWORK_PORT_PASS.ip,
                lib.tests.mock_midonet_api.IPNETWORK_PORT_PASS.prefixlen,
                lib.tests.mocks.midonet.UUID_PORT_ID_PASS,
                lib.tests.mocks.midonet.UUID_ROUTER_ID_PASS,
            ),
            charmhelpers.core.hookenv.DEBUG,
        )

    def test_delete(self):
        self.mocked_log.reset_mock()
        lib.tests.mock_midonet_api.mock_router_port_list_pass()
        lib.tests.mock_midonet_api.mock_port_delete()

        port = lib.midonet_api.router_get_port(
            router=self.provider_router,
            port_address=str(lib.tests.mock_midonet_api.IPNETWORK_PORT_PASS),
        )
        lib.midonet_api.port_delete(port=port)

        asserted_log_calls = [
            mock.call(
                lib.log.MIDONET_ROUTER_PORT_FOUND.format(
                    lib.tests.mock_midonet_api.IPNETWORK_PORT_PASS.ip,
                    lib.tests.mock_midonet_api.IPNETWORK_PORT_PASS.prefixlen,
                    lib.tests.mocks.midonet.UUID_PORT_ID_PASS,
                    lib.tests.mocks.midonet.UUID_ROUTER_ID_PASS,
                ),
                charmhelpers.core.hookenv.DEBUG,
            ),
            mock.call(
                lib.log.MIDONET_PORT_DELETED.format(
                    lib.tests.mock_midonet_api.IPNETWORK_PORT_PASS.ip,
                    lib.tests.mock_midonet_api.IPNETWORK_PORT_PASS.prefixlen,
                    lib.tests.mocks.midonet.UUID_PORT_ID_PASS,
                    lib.tests.mocks.midonet.UUID_ROUTER_ID_PASS,
                ),
                charmhelpers.core.hookenv.DEBUG,
            )
        ]

        assert self.mocked_log.mock_calls == asserted_log_calls
