from __future__ import absolute_import
from __future__ import division
from __future__ import print_function
from __future__ import unicode_literals

import charmhelpers.core.hookenv
import httpretty
import mock
import netaddr

import lib.log
import lib.midonet_api
import lib.misc
import lib.tests.mock_midonet_api
import lib.tests.mocks.midonet


class TestRoutersRoute(object):
    def setup(self):
        self.patcher_log = mock.patch(
            'charmhelpers.core.hookenv.log',
        )
        self.mocked_log = self.patcher_log.start()

        httpretty.enable()
        httpretty.HTTPretty.allow_net_connect = False

        lib.tests.mock_midonet_api.mock_login()
        lib.tests.mock_midonet_api.mock_root()
        lib.tests.mock_midonet_api.mock_system_state()
        lib.tests.mock_midonet_api.mock_routers_list_pass()
        lib.tests.mock_midonet_api.mock_router_port_list_pass()

        self.provider_router = lib.midonet_api.get_provider_router(
            midonet_client=lib.tests.mocks.midonet.get_bogus_midonetclient(),
        )
        self.port = lib.midonet_api.router_get_port(
            router=self.provider_router,
            port_address=str(lib.tests.mock_midonet_api.IPNETWORK_PORT_PASS),
        )

    def teardown(self):
        httpretty.disable()
        httpretty.reset()

        self.patcher_log.stop()

    def test_list_pass(self):
        self.mocked_log.reset_mock()
        lib.tests.mock_midonet_api.mock_routers_route_list_pass()

        route = lib.midonet_api.router_get_route(
            router=self.provider_router,
            src_net=str(lib.tests.mock_midonet_api.IPNETWORK_SRC_NET_PASS),
            dst_net=str(lib.tests.mock_midonet_api.IPNETWORK_DST_NET_PASS),
            next_hop_port=self.port,
            route_type='Normal',
        )

        _src_net = netaddr.IPNetwork('{}/{}'.format(
            route.get_src_network_addr(),
            route.get_src_network_length(),
        ))
        _dst_net = netaddr.IPNetwork('{}/{}'.format(
            route.get_dst_network_addr(),
            route.get_dst_network_length(),
        ))

        assert _src_net == lib.tests.mock_midonet_api.IPNETWORK_SRC_NET_PASS
        assert _dst_net == lib.tests.mock_midonet_api.IPNETWORK_DST_NET_PASS
        assert route.get_next_hop_port() == str(
            lib.tests.mocks.midonet.UUID_PORT_ID_PASS,
        )
        assert route.get_type() == 'Normal'
        assert route.get_id() == str(
            lib.tests.mocks.midonet.UUID_ROUTE_ID_PASS,
        )
        assert route.get_router_id() == str(
            lib.tests.mocks.midonet.UUID_ROUTER_ID_PASS,
        )
        self.mocked_log.assert_called_once_with(
            lib.log.MIDONET_ROUTER_ROUTE_FOUND.format(
                lib.tests.mock_midonet_api.IPNETWORK_SRC_NET_PASS,
                lib.tests.mock_midonet_api.IPNETWORK_DST_NET_PASS,
                lib.tests.mocks.midonet.UUID_PORT_ID_PASS,
                None,
                'Normal',
                lib.tests.mocks.midonet.UUID_ROUTE_ID_PASS,
                lib.tests.mocks.midonet.UUID_ROUTER_ID_PASS,
            ),
            charmhelpers.core.hookenv.DEBUG,
        )

    def test_list_fail(self):
        self.mocked_log.reset_mock()
        lib.tests.mock_midonet_api.mock_routers_route_list_fail()

        route = lib.midonet_api.router_get_route(
            router=self.provider_router,
            src_net=str(lib.tests.mock_midonet_api.IPNETWORK_SRC_NET_PASS),
            dst_net=str(lib.tests.mock_midonet_api.IPNETWORK_DST_NET_PASS),
            next_hop_port=self.port,
            route_type='Normal',
        )

        assert route is None
        self.mocked_log.assert_called_once_with(
            lib.log.MIDONET_ROUTER_ROUTE_NOT_FOUND.format(
                lib.tests.mock_midonet_api.IPNETWORK_SRC_NET_PASS,
                lib.tests.mock_midonet_api.IPNETWORK_DST_NET_PASS,
                lib.tests.mocks.midonet.UUID_PORT_ID_PASS,
                None,
                'Normal',
                lib.tests.mocks.midonet.UUID_ROUTER_ID_PASS,
            ),
            charmhelpers.core.hookenv.DEBUG,
        )

    def test_add(self):
        self.mocked_log.reset_mock()
        lib.tests.mock_midonet_api.mock_routers_route_list_fail()
        lib.tests.mock_midonet_api.mock_routers_route_add()
        lib.tests.mock_midonet_api.mock_route_get()

        route = lib.midonet_api.router_add_route(
            router=self.provider_router,
            src_net=lib.tests.mock_midonet_api.IPNETWORK_SRC_NET_PASS,
            dst_net=lib.tests.mock_midonet_api.IPNETWORK_DST_NET_PASS,
            next_hop_port=self.port,
            route_type='Normal',
        )

        _src_net = netaddr.IPNetwork('{}/{}'.format(
            route.get_src_network_addr(),
            route.get_src_network_length(),
        ))
        _dst_net = netaddr.IPNetwork('{}/{}'.format(
            route.get_dst_network_addr(),
            route.get_dst_network_length(),
        ))

        assert _src_net == lib.tests.mock_midonet_api.IPNETWORK_SRC_NET_PASS
        assert _dst_net == lib.tests.mock_midonet_api.IPNETWORK_DST_NET_PASS
        assert route.get_next_hop_port() == str(
            lib.tests.mocks.midonet.UUID_PORT_ID_PASS,
        )
        assert route.get_type() == 'Normal'
        assert route.get_id() == str(
            lib.tests.mocks.midonet.UUID_ROUTE_ID_PASS,
        )
        assert route.get_router_id() == str(
            lib.tests.mocks.midonet.UUID_ROUTER_ID_PASS,
        )
        self.mocked_log.assert_called_with(
            lib.log.MIDONET_ROUTER_ROUTE_CREATED.format(
                lib.tests.mock_midonet_api.IPNETWORK_SRC_NET_PASS,
                lib.tests.mock_midonet_api.IPNETWORK_DST_NET_PASS,
                lib.tests.mocks.midonet.UUID_PORT_ID_PASS,
                None,
                'Normal',
                lib.tests.mocks.midonet.UUID_ROUTE_ID_PASS,
                lib.tests.mocks.midonet.UUID_ROUTER_ID_PASS,
            ),
            charmhelpers.core.hookenv.DEBUG,
        )

    def test_delete(self):
        self.mocked_log.reset_mock()
        lib.tests.mock_midonet_api.mock_routers_route_list_pass()
        lib.tests.mock_midonet_api.mock_route_delete()

        route = lib.midonet_api.router_get_route(
            router=self.provider_router,
            src_net=str(lib.tests.mock_midonet_api.IPNETWORK_SRC_NET_PASS),
            dst_net=str(lib.tests.mock_midonet_api.IPNETWORK_DST_NET_PASS),
            next_hop_port=self.port,
            route_type='Normal',
        )
        lib.midonet_api.route_delete(route=route)

        asserted_log_calls = [
            mock.call(
                lib.log.MIDONET_ROUTER_ROUTE_FOUND.format(
                    lib.tests.mock_midonet_api.IPNETWORK_SRC_NET_PASS,
                    lib.tests.mock_midonet_api.IPNETWORK_DST_NET_PASS,
                    lib.tests.mocks.midonet.UUID_PORT_ID_PASS,
                    None,
                    'Normal',
                    lib.tests.mocks.midonet.UUID_ROUTE_ID_PASS,
                    lib.tests.mocks.midonet.UUID_ROUTER_ID_PASS,
                ),
                charmhelpers.core.hookenv.DEBUG,
            ),
            mock.call(
                lib.log.MIDONET_ROUTE_DELETED.format(
                    lib.tests.mock_midonet_api.IPNETWORK_SRC_NET_PASS,
                    lib.tests.mock_midonet_api.IPNETWORK_DST_NET_PASS,
                    lib.tests.mocks.midonet.UUID_PORT_ID_PASS,
                    None,
                    'Normal',
                    lib.tests.mocks.midonet.UUID_ROUTE_ID_PASS,
                    lib.tests.mocks.midonet.UUID_ROUTER_ID_PASS,
                ),
                charmhelpers.core.hookenv.DEBUG,
            ),
        ]

        assert self.mocked_log.mock_calls == asserted_log_calls
