from __future__ import absolute_import
from __future__ import division
from __future__ import print_function
from __future__ import unicode_literals

import charmhelpers.core.hookenv
import mock
import netaddr
import nose.tools

import lib.host
import lib.misc
import lib.tests.mock_host


class TestSetupVethPair(object):
    def setup(self):
        self.patcher_log = mock.patch(
            'charmhelpers.core.hookenv.log',
        )
        self.mocked_log = self.patcher_log.start()

        self.patcher_render = mock.patch(
            'charmhelpers.core.templating.render',
        )
        self.mocked_render = self.patcher_render.start()

        self.patcher_check_call = mock.patch(
            'subprocess.check_call',
        )
        self.mocked_check_call = self.patcher_check_call.start()

    def teardown(self):
        self.patcher_log.stop()
        self.patcher_render.stop()
        self.patcher_check_call.stop()

    def test_interface_common(self):
        lib.host.static_setup_veth_pair()

        nose.tools.assert_equal(
            self.mocked_log.mock_calls[0],
            mock.call(
                lib.log.HOST_SETUP_VETH_PAIR_START.format(
                    lib.misc.STATIC_BRIDGE_IF_NAME,
                    lib.misc.STATIC_PROVIDER_ROUTER_IF_NAME,
                ),
                charmhelpers.core.hookenv.DEBUG,
            ),
        )

        asserted_calls = [
            mock.call(
                source='etc/network/interfaces.d/veth.cfg',
                target='/etc/network/interfaces.d/{}.cfg'.format(
                    lib.misc.STATIC_BRIDGE_IF_NAME,
                ),
                context={
                    'this_interface': lib.misc.STATIC_BRIDGE_IF_NAME,
                    'that_interface': lib.misc.STATIC_PROVIDER_ROUTER_IF_NAME,
                },
            ),
            mock.call(
                source='etc/network/interfaces.d/veth.cfg',
                target='/etc/network/interfaces.d/{}.cfg'.format(
                    lib.misc.STATIC_PROVIDER_ROUTER_IF_NAME,
                ),
                context={
                    'this_interface': lib.misc.STATIC_PROVIDER_ROUTER_IF_NAME,
                    'that_interface': lib.misc.STATIC_BRIDGE_IF_NAME,
                },
            ),
        ]
        nose.tools.assert_equal(self.mocked_render.mock_calls, asserted_calls)

    def test_interface_down(self):
        self.mocked_check_call.side_effect = (
            lib.tests.mock_host.side_effect_interface_down
        )

        lib.host.static_setup_veth_pair()

        asserted_calls = [
            mock.call(
                ['ifquery', '--state', lib.misc.STATIC_BRIDGE_IF_NAME]
            ),
            mock.call(
                ['ifup', lib.misc.STATIC_BRIDGE_IF_NAME]
            ),
            mock.call(
                ['ifquery', '--state', lib.misc.STATIC_PROVIDER_ROUTER_IF_NAME]
            ),
            mock.call(
                ['ifup', lib.misc.STATIC_PROVIDER_ROUTER_IF_NAME]
            ),
        ]
        nose.tools.assert_equal(
            self.mocked_check_call.mock_calls,
            asserted_calls,
        )

        asserted_calls = [
            mock.call(
                lib.log.HOST_IFUP.format(
                    lib.misc.STATIC_BRIDGE_IF_NAME,
                ),
                charmhelpers.core.hookenv.DEBUG,
            ),
            mock.call(
                lib.log.HOST_IFUP.format(
                    lib.misc.STATIC_PROVIDER_ROUTER_IF_NAME,
                ),
                charmhelpers.core.hookenv.DEBUG,
            ),
        ]
        nose.tools.assert_equal(
            self.mocked_log.mock_calls[-2:],
            asserted_calls,
        )

    def test_interface_up(self):
        self.mocked_check_call.side_effect = (
            lib.tests.mock_host.side_effect_interface_up
        )

        lib.host.static_setup_veth_pair()

        asserted_calls = [
            mock.call(
                ['ifquery', '--state', lib.misc.STATIC_BRIDGE_IF_NAME]
            ),
            mock.call(
                ['ifquery', '--state', lib.misc.STATIC_PROVIDER_ROUTER_IF_NAME]
            ),
        ]
        nose.tools.assert_equal(
            self.mocked_check_call.mock_calls,
            asserted_calls,
        )


class TestSetupBridge(object):
    def setup(self):
        self.rd_config = {
            'static-linux-bridge-address': '192.0.2.1/30',
            'static-provider-router-address': '192.0.2.2/30',
            'static-forwarding-network': '10.20.30.40/24',
        }

        self.patcher_log = mock.patch(
            'charmhelpers.core.hookenv.log',
        )
        self.mocked_log = self.patcher_log.start()

        self.patcher_render = mock.patch(
            'charmhelpers.core.templating.render',
        )
        self.mocked_render = self.patcher_render.start()

        self.patcher_check_call = mock.patch(
            'subprocess.check_call',
        )
        self.mocked_check_call = self.patcher_check_call.start()

    def teardown(self):
        self.patcher_log.stop()
        self.patcher_render.stop()
        self.patcher_check_call.stop()

    def test_setup_bridge_common(self):
        bridge_address = netaddr.IPNetwork(
            self.rd_config['static-linux-bridge-address'],
        )
        provider_router_address = netaddr.IPNetwork(
            self.rd_config['static-provider-router-address'],
        )
        floating_ip_network = netaddr.IPNetwork(
            self.rd_config['static-forwarding-network'],
        )

        lib.host.static_setup_bridge(self.rd_config)

        nose.tools.assert_equal(
            self.mocked_log.mock_calls[0],
            mock.call(
                lib.log.HOST_SETUP_STATIC_BRIDGE_START,
                charmhelpers.core.hookenv.DEBUG,
            ),
        )
        self.mocked_render.assert_called_once_with(
            source='etc/network/interfaces.d/bridge.cfg',
            target='/etc/network/interfaces.d/{}.cfg'.format(
                lib.misc.STATIC_BRIDGE_NAME,
            ),
            context={
                'bridge_name': lib.misc.STATIC_BRIDGE_NAME,
                'bridge_if_name': lib.misc.STATIC_BRIDGE_IF_NAME,
                'bridge_address': str(bridge_address.ip),
                'bridge_netmask': str(bridge_address.netmask),
                'floating_ip_network': str(floating_ip_network),
                'provider_router_address': str(provider_router_address.ip),
            },
        )

    def test_setup_bridge_down(self):
        self.mocked_check_call.side_effect = (
            lib.tests.mock_host.side_effect_interface_down
        )

        lib.host.static_setup_bridge(self.rd_config)

        asserted_calls = [
            mock.call(['ifquery', '--state', lib.misc.STATIC_BRIDGE_NAME]),
            mock.call(['ifup', lib.misc.STATIC_BRIDGE_NAME]),
        ]
        nose.tools.assert_equal(
            self.mocked_check_call.mock_calls,
            asserted_calls,
        )

        nose.tools.assert_equal(
            self.mocked_log.mock_calls[-1],
            mock.call(
                lib.log.HOST_IFUP.format(
                    lib.misc.STATIC_BRIDGE_NAME,
                ),
                charmhelpers.core.hookenv.DEBUG,
            ),
        )

    def test_setup_bridge_up(self):
        self.mocked_check_call.side_effect = (
            lib.tests.mock_host.side_effect_interface_up
        )

        lib.host.static_setup_bridge(self.rd_config)

        self.mocked_check_call.assert_called_once_with(
            ['ifquery', '--state', lib.misc.STATIC_BRIDGE_NAME],
        )


class TestSysctl(object):
    def setup(self):
        self.patcher_check_output = mock.patch(
            'subprocess.check_output',
        )
        self.mocked_check_output = self.patcher_check_output.start()

        self.patcher_check_call = mock.patch(
            'subprocess.check_call',
        )
        self.mocked_check_call = self.patcher_check_call.start()

    def teardown(self):
        self.patcher_check_output.stop()
        self.patcher_check_call.stop()

    def test_sysctl_same(self):
        self.mocked_check_output.return_value = (
            lib.tests.mock_host.SYSCTL_FAKE_VALUE_SAME
        )

        lib.host.sysctl(
            lib.tests.mock_host.SYSCTL_FAKE_KEY,
            lib.tests.mock_host.SYSCTL_FAKE_VALUE_SAME,
        )

        self.mocked_check_output.assert_called_once_with(
            ['sysctl', '-b', lib.tests.mock_host.SYSCTL_FAKE_KEY]
        )
        self.mocked_check_call.assert_not_called()

    def test_sysctl_different(self):
        self.mocked_check_output.return_value = (
            lib.tests.mock_host.SYSCTL_FAKE_VALUE_DIFFERENT
        )

        lib.host.sysctl(
            lib.tests.mock_host.SYSCTL_FAKE_KEY,
            lib.tests.mock_host.SYSCTL_FAKE_VALUE_SAME,
        )

        self.mocked_check_output.assert_called_once_with(
            ['sysctl', '-b', lib.tests.mock_host.SYSCTL_FAKE_KEY]
        )
        self.mocked_check_call.assert_called_once_with([
            'sysctl',
            '-w',
            '='.join((
                lib.tests.mock_host.SYSCTL_FAKE_KEY,
                lib.tests.mock_host.SYSCTL_FAKE_VALUE_SAME,
            )),
        ])


class TestSetupNetfilterMasquerade(object):
    def setup(self):
        self.rd_config = {
            'static-forwarding-network': '10.20.30.0/24',
            'static-outgoing-interface-name': 'some-interface',
        }

        self.patcher_log = mock.patch(
            'charmhelpers.core.hookenv.log',
        )
        self.mocked_log = self.patcher_log.start()

        self.patcher_call = mock.patch('subprocess.call')
        self.mocked_call = self.patcher_call.start()

        self.patcher_check_call = mock.patch('subprocess.check_call')
        self.mocked_check_call = self.patcher_check_call.start()

    def teardown(self):
        self.patcher_check_call.stop()
        self.patcher_call.stop()
        self.patcher_log.stop()

    def test_rule_not_present(self):
        self.mocked_call.return_value = 1

        lib.host.setup_netfilter_masquerade(self.rd_config)

        self.mocked_call.assert_called_once_with([
            'iptables',
            '-t', 'nat',
            '-C', 'POSTROUTING',
            '-o', self.rd_config['static-outgoing-interface-name'],
            '-s', self.rd_config['static-forwarding-network'],
            '-j', 'MASQUERADE',
            '-m', 'comment',
            '--comment', 'JUJU-CHARM-MIDONET-GATEWAY-MANAGED',
        ])
        self.mocked_check_call.assert_called_once_with([
            'iptables',
            '-t', 'nat',
            '-I', 'POSTROUTING',
            '-o', self.rd_config['static-outgoing-interface-name'],
            '-s', self.rd_config['static-forwarding-network'],
            '-j', 'MASQUERADE',
            '-m', 'comment',
            '--comment', 'JUJU-CHARM-MIDONET-GATEWAY-MANAGED',
        ])
        self.mocked_log.assert_called_once_with(
            lib.log.HOST_SETUP_MASQUERADING_NAT_RULE_CREATED,
            charmhelpers.core.hookenv.DEBUG,
        )

    def test_rule_present(self):
        self.mocked_call.return_value = 0

        lib.host.setup_netfilter_masquerade(self.rd_config)

        self.mocked_call.assert_called_once_with([
            'iptables',
            '-t', 'nat',
            '-C', 'POSTROUTING',
            '-o', self.rd_config['static-outgoing-interface-name'],
            '-s', self.rd_config['static-forwarding-network'],
            '-j', 'MASQUERADE',
            '-m', 'comment',
            '--comment', 'JUJU-CHARM-MIDONET-GATEWAY-MANAGED',
        ])
        self.mocked_log.assert_called_once_with(
            lib.log.HOST_SETUP_MASQUERADING_NAT_RULE_FOUND,
            charmhelpers.core.hookenv.DEBUG,
        )


class TestSetupNetfilterForward(object):
    def setup(self):
        self.rd_config = {
            'static-forwarding-network': '10.20.30.0/24',
        }

        self.patcher_log = mock.patch(
            'charmhelpers.core.hookenv.log',
        )
        self.mocked_log = self.patcher_log.start()

        self.patcher_call = mock.patch('subprocess.call')
        self.mocked_call = self.patcher_call.start()

        self.patcher_check_call = mock.patch('subprocess.check_call')
        self.mocked_check_call = self.patcher_check_call.start()

    def teardown(self):
        self.patcher_check_call.stop()
        self.patcher_call.stop()
        self.patcher_log.stop()

    def test_rule_not_present(self):
        self.mocked_call.return_value = 1

        lib.host.setup_netfilter_forward(self.rd_config)

        self.mocked_call.assert_called_once_with([
            'iptables',
            '-C', 'FORWARD',
            '-s', self.rd_config['static-forwarding-network'],
            '-j', 'ACCEPT',
            '-m', 'comment',
            '--comment', 'JUJU-CHARM-MIDONET-GATEWAY-MANAGED',
        ])
        self.mocked_check_call.assert_called_once_with([
            'iptables',
            '-I', 'FORWARD',
            '-s', self.rd_config['static-forwarding-network'],
            '-j', 'ACCEPT',
            '-m', 'comment',
            '--comment', 'JUJU-CHARM-MIDONET-GATEWAY-MANAGED',
        ])
        self.mocked_log.assert_called_once_with(
            lib.log.HOST_SETUP_MASQUERADING_FORWARD_RULE_CREATED,
            charmhelpers.core.hookenv.DEBUG,
        )

    def test_rule_present(self):
        self.mocked_call.return_value = 0

        lib.host.setup_netfilter_forward(self.rd_config)

        self.mocked_call.assert_called_once_with([
            'iptables',
            '-C', 'FORWARD',
            '-s', self.rd_config['static-forwarding-network'],
            '-j', 'ACCEPT',
            '-m', 'comment',
            '--comment', 'JUJU-CHARM-MIDONET-GATEWAY-MANAGED',
        ])
        self.mocked_log.assert_called_once_with(
            lib.log.HOST_SETUP_MASQUERADING_FORWARD_RULE_FOUND,
            charmhelpers.core.hookenv.DEBUG,
        )
